package com.s5gi.hud;

import com.mojang.blaze3d.systems.RenderSystem;
import com.s5gi.networking.payloads.S2C.HudRendererTimeUpdatePayload;
import net.fabricmc.fabric.api.client.networking.v1.ClientPlayNetworking;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.render.*;
import net.minecraft.util.Identifier;

import static com.s5gi.Lacess.MOD_ID;

public class HudRenderer {

    public static void handleHudRendererTimeUpdateNetworking(HudRendererTimeUpdatePayload payload, ClientPlayNetworking.Context context) {
        context.client().execute(() -> {
            switch (HudRendererTimeUpdatePayload.vars.getVar(payload.var())) {
                case CHORUS_DUST_OVERLAY: {
                    ChorusDustOverlay.startTime = System.currentTimeMillis();
                }
            }
        });
    }

    public static void drawHud(DrawContext context, RenderTickCounter counter) {
        ChorusDustOverlay.drawChorusDustHudOverlay(context, counter);
    }



    private static class ChorusDustOverlay {
        private static final int FADE_START_DELAY = 2200; // Delay before fading starts in milliseconds
        private static final int DISPLAY_DURATION = 3000;
        private static final Identifier image_id = Identifier.of(MOD_ID, "textures/gui/chorus_dust_overlay.png");


        private static long startTime = -1;
        private static void drawChorusDustHudOverlay(DrawContext context, RenderTickCounter counter) {
            if (startTime >= 0) {
                long currentTime = System.currentTimeMillis();
                long elapsedTime = currentTime - startTime;

                if (elapsedTime >= DISPLAY_DURATION) {
                    startTime = -1; // Reset startTime to stop rendering
                    return;
                }

                MinecraftClient client = MinecraftClient.getInstance();
                int screenWidth = client.getWindow().getScaledWidth();
                int screenHeight = client.getWindow().getScaledHeight();

                float opacity = getOpacity(elapsedTime);

                if (MinecraftClient.getInstance().currentScreen == null) {
                    RenderSystem.enableBlend();
                    RenderSystem.setShaderTexture(5, image_id);
                    RenderSystem.defaultBlendFunc();
                    RenderSystem.setShaderColor(1f, 1f, 1f, opacity);
                    //stupidest shit known to man kind
                    context.drawTexture(RenderLayer::getGuiTexturedOverlay, image_id, 0, 0, 0, 0, screenWidth, screenHeight, screenWidth, screenHeight);
                    //end of stupidest shit known ato man kind
                    RenderSystem.disableBlend();
                } else {
                    RenderSystem.setShaderColor(1f, 1f, 1f, 1f);
                    RenderSystem.disableBlend();
                }

            }
        }

        private static float getOpacity(long elapsedTime) {
            float opacity;
            if (elapsedTime < FADE_START_DELAY) {
                opacity = 1.0f; // Full opacity during the delay
            } else {
                // Calculate fade-out opacity
                long fadeElapsedTime = elapsedTime - FADE_START_DELAY;
                opacity = 1.0f - ((float) fadeElapsedTime / (DISPLAY_DURATION - FADE_START_DELAY));
                // Ensure opacity doesn't go below 0
                opacity = Math.max(opacity, 0.0f);
            }
            return opacity;
        }
    }
}
